#!/usr/bin/env python

from __future__ import print_function
import sys
import boto.ec2
import requests
import dateutil.parser


def iso8601_to_py(ts):
  """
  This method converts the ISO-8601 timestamp string to a python datetime.
  """
  return dateutil.parser.parse(ts)


def normalize_az(api_url, api_token, customer_access_key_id,
    customer_secret_access_key, region):
  """
  This method is supposed to be called by the customer using his
  aws_access_key_id and aws_secret_access_key.

  It takes a region and returns the mapping from customer's availability zone
  to provider's normalized availability zones.
  """

  print("Getting history from provider...", file=sys.stderr)

  payload = {'region': region}
  history_url = api_url.rstrip("/") + "/get_history"
  headers = {'X-Api-Token': api_token}
  response = requests.get(history_url, params=payload, headers=headers)
  if response.status_code != 200:
    print("Getting history from provider failed.", file=sys.stderr)
    print("Status Code: %s" % response.status_code, file=sys.stderr)
    print(response.text, file=sys.stderr)
    return None
  provider_history = response.json()

  customer_ec2 = boto.ec2.connect_to_region(region,
      aws_access_key_id=customer_access_key_id,
      aws_secret_access_key=customer_secret_access_key)

  if customer_ec2 is None:
    print("Invalid region name provided.", file=sys.stderr)
    return None

  try:
    print("Getting customer history from AWS...", file=sys.stderr)

    csph = customer_ec2.get_spot_price_history(
        start_time=provider_history['start_time'],
        end_time=provider_history['end_time'],
        filters={"product-description":["Linux/UNIX", "Linux/UNIX (Amazon VPC)"]})
  except boto.exception.EC2ResponseError as e:
    print("Getting customer history from AWS failed.", file=sys.stderr)
    print("Status Code: %s" % e.status, file=sys.stderr)
    print("Reason: %s" % e.reason, file=sys.stderr)
    print("Body: %s" % e.body, file=sys.stderr)
    return None

  print("Fetching history complete.", file=sys.stderr)

  # Find out if the customer's account-region supports both EC2-classic and VPC
  # or only VPC. Please note that this hack will only work if the provider's
  # account-region supports both EC2-classic and VPC. In that case, it would
  # correctly send the product_description as "Linux/UNIX (Amazon VPC)" in case
  # of VPC. If the provider's account-region is VPC only, then we are toast.
  # In that case, some changes are needed on the server side to tell the client
  # that only compare with your vpc history. Fortunately, Qubole's provider
  # acccount supports both.
  print("Finding out if the customer account is VPC only...", file=sys.stderr)
  acc_attributes = customer_ec2.describe_account_attributes()
  for attribute in acc_attributes:
    if attribute.attribute_name == "supported-platforms":
      if 'EC2' in attribute.attribute_values:
        print("Customer's account supports both EC2-classic and VPC.", file=sys.stderr)
        customer_product_description = "Linux/UNIX"
      else:
        print("Customer's account supports only VPC.", file=sys.stderr)
        customer_product_description = "Linux/UNIX (Amazon VPC)"
      break

  results = {}

  mapping = {}
  for rsp in provider_history['history']:
    timestamp = iso8601_to_py(rsp['timestamp'])
    if (timestamp, rsp['instance_type'], rsp['price'], rsp['product_description']) in mapping:
      # We only use unique values for comparison
      del mapping[timestamp, rsp['instance_type'], rsp['price'], rsp['product_description']]
    else:
      mapping[timestamp, rsp['instance_type'], rsp['price'], rsp['product_description']] = rsp['availability_zone']

  for csp in csph:
    timestamp = iso8601_to_py(csp.timestamp)
    if (timestamp, csp.instance_type, csp.price, customer_product_description) in mapping:
      # We got a similar tuple using reference credentials, so we can match.
      ref_az = mapping[timestamp, csp.instance_type, csp.price, customer_product_description]
      if csp.availability_zone in results:
        if results[csp.availability_zone][0] == ref_az:
          confirmations = results[csp.availability_zone][1] + 1
          results[csp.availability_zone] = ref_az, confirmations
        else:
          # We have a negative confirmation! Can this happen?
          pass
      else:
        results[csp.availability_zone] = ref_az, 1

  output = {}
  for key in results.keys():
    if results[key][1] < 10:
      # Number of confirmations are too low, probably not a right mapping.
      # Remove it. These type of instances can occur when there is a AZ in
      # customer account which is not there in reference account.
      del results[key]
    else:
      output[key] = results[key][0]

  return output


if __name__ == "__main__":


  customer_access_key_id = raw_input("Enter aws_access_key_id for customer account: ")
  customer_secret_access_key = raw_input("Enter aws_secret_access_key for customer account: ")
  api_token = raw_input("Enter the api token for spot prediction service: ")
  api_url = raw_input("Enter the api url for spot prediction service: ")
  region = raw_input("Enter region: ")

  results = normalize_az(
      api_url,
      api_token,
      customer_access_key_id,
      customer_secret_access_key,
      region)

  import json
  print(json.dumps(results, indent=4))
